@@ -1,5 +1,6 @@
 import argparse
 import imp
+import locale
 import os
 import select
 import sys
@@ -33,8 +34,8 @@ def print_line(line):
     """
     Print given line to stdout (i3bar).
     """
-    sys.stdout.write('{}\n'.format(line))
-    sys.stdout.flush()
+    sys.__stdout__.write('{}\n'.format(line))
+    sys.__stdout__.flush()
 
 
 class IOPoller:
@@ -58,6 +59,9 @@ class IOPoller:
         poll_result = self.poller.poll(timeout)
         if poll_result:
             line = self.io.readline().strip()
+            if self.io == sys.stdin and line == '[':
+                # skip first event line wrt issue #19
+                line = self.io.readline().strip()
             try:
                 # python3 compatibility code
                 line = line.decode()
@@ -84,6 +88,8 @@ class I3status(Thread):
         self.last_output_ts = None
         self.last_prefix = None
         self.lock = lock
+        self.json_list = None
+        self.json_list_ts = None
 
     def i3status_config_reader(self):
         """
@@ -136,25 +142,44 @@ class I3status(Thread):
 
         return config
 
-    def transform(self, delta):
+    def adjust_time(self, delta):
         """
-        Apply integrated transformations:
-            - adjust the 'time' object so that it's updated at interval seconds
+        Adjust the 'time' object so that it's updated at interval seconds
         """
-        json_list = deepcopy(self.last_output)
+        json_list = deepcopy(self.json_list)
         try:
             time_format = self.config['time_format']
             for item in json_list:
                 if item['name'] in ['time', 'tztime']:
-                    date = datetime.strptime(item['full_text'], time_format)
+                    i3status_time = item['full_text']
+                    # add mendatory items in i3status time format wrt issue #18
+                    for fmt in ['%Y', '%m', '%d']:
+                        if not fmt in time_format:
+                            time_format = '{} {}'.format(time_format, fmt)
+                            i3status_time = '{} {}'.format(
+                                i3status_time, datetime.now().strftime(fmt)
+                            )
+                    date = datetime.strptime(i3status_time, time_format)
                     date += delta
-                    item['full_text'] = date.strftime(time_format)
+                    item['full_text'] = date.strftime(
+                        self.config['time_format']
+                    )
                     item['transformed'] = True
         except Exception:
             err = sys.exc_info()[1]
-            syslog(LOG_ERR, "transformation failed ({})".format(err))
+            syslog(LOG_ERR, "i3status adjust_time failed ({})".format(err))
         return json_list
 
+    def update_json_list(self):
+        """
+        Copy the last json list output from i3status so that any module
+        can modify it without altering the original output.
+        This is done so that any module's alteration of a i3status output json
+        will not be overwritten when the next i3status output gets polled.
+        """
+        self.json_list = deepcopy(self.last_output)
+        self.json_list_ts = deepcopy(self.last_output_ts)
+
     def run(self):
         """
         Spawn i3status and poll its output.
@@ -175,7 +200,14 @@ class I3status(Thread):
             while self.lock.is_set():
                 line = self.poller_inp.readline(timeout)
                 if line:
-                    if not line.startswith(','):
+                    if line.startswith('[{'):
+                        with jsonify(line) as (prefix, json_list):
+                            self.last_output = json_list
+                            self.last_output_ts = datetime.utcnow()
+                            self.last_prefix = ','
+                            self.update_json_list()
+                        print_line(line)
+                    elif not line.startswith(','):
                         if 'version' in line:
                             header = loads(line)
                             header.update({'click_events': True})
@@ -233,6 +265,22 @@ class Events(Thread):
             if self.config['debug']:
                 syslog(LOG_INFO, 'dispatching default event {}'.format(event))
 
+    def i3bar_click_events_module(self):
+        """
+        Detect the presence of the special i3bar_click_events.py module.
+
+        When py3status detects a module named 'i3bar_click_events.py',
+        it will dispatch i3status click events to this module so you can catch
+        them and trigger any function call based on the event.
+        """
+        for module in self.modules:
+            if not module.click_events:
+                continue
+            if module.module_name == 'i3bar_click_events.py':
+                return module
+        else:
+            return False
+
     def run(self):
         """
         Wait for an i3bar JSON event, then find the right module to dispatch
@@ -247,7 +295,7 @@ class Events(Thread):
         """
         while self.lock.is_set():
             event = self.poller_inp.readline()
-            if not event or event == '[':
+            if not event:
                 sleep(0.20)
                 continue
             try:
@@ -257,6 +305,7 @@ class Events(Thread):
 
                     # setup default action on button 2 press
                     default_event = False
+                    dispatched = False
                     if 'button' in event and event['button'] == 2:
                         default_event = True
 
@@ -272,10 +321,23 @@ class Events(Thread):
                                 if 'instance' in event:
                                     if event['instance'] == obj['instance']:
                                         self.dispatch(module, obj, event)
+                                        dispatched = True
                                         break
                                 else:
                                     self.dispatch(module, obj, event)
+                                    dispatched = True
                                     break
+
+                    # fall back to i3bar_click_events.py module if present
+                    if not dispatched:
+                        module = self.i3bar_click_events_module()
+                        if module:
+                            if self.config['debug']:
+                                syslog(
+                                    LOG_INFO,
+                                    'dispatching event to i3bar_click_events'
+                                )
+                            self.dispatch(module, obj, event)
             except Exception:
                 err = sys.exc_info()[1]
                 syslog(LOG_WARNING, 'event failed ({})'.format(err))
@@ -287,7 +349,7 @@ class Module(Thread):
     It is reponsible for executing it every given interval and
     caching its output based on user will.
     """
-    def __init__(self, lock, config, include_path, f_name, i3_conf, i3_json):
+    def __init__(self, lock, config, include_path, f_name, i3_thread):
         """
         We need quite some stuff to occupy ourselves don't we ?
         """
@@ -295,12 +357,12 @@ class Module(Thread):
         self.click_events = False
         self.config = config
         self.has_kill = False
-        self.i3status_conf = i3_conf
-        self.i3status_json = i3_json
+        self.i3status_thread = i3_thread
         self.last_output = []
         self.lock = lock
         self.methods = {}
         self.module_class = None
+        self.module_name = f_name
         #
         self.load_methods(include_path, f_name)
 
@@ -370,7 +432,7 @@ class Module(Thread):
             syslog(
                 LOG_INFO,
                 'module {} click_events={} has_kill={} methods={}'.format(
-                    f_name,
+                    self.module_name,
                     self.click_events,
                     self.has_kill,
                     self.methods.keys()
@@ -384,8 +446,8 @@ class Module(Thread):
         try:
             click_method = getattr(self.module_class, 'on_click')
             click_method(
-                self.i3status_json,
-                self.i3status_conf,
+                self.i3status_thread.json_list,
+                self.i3status_thread.config,
                 event
             )
         except Exception:
@@ -401,9 +463,6 @@ class Module(Thread):
         We will execute the 'kill' method of the module when we terminate.
         """
         while self.lock.is_set():
-            # don't be hasty mate, let's take it easy for now
-            sleep(self.config['interval'])
-
             # execute each method of this module
             for meth, obj in self.methods.items():
                 my_method = self.methods[meth]
@@ -420,8 +479,8 @@ class Module(Thread):
                     # execute method and get its output
                     method = getattr(self.module_class, meth)
                     position, result = method(
-                        self.i3status_json,
-                        self.i3status_conf
+                        self.i3status_thread.json_list,
+                        self.i3status_thread.config
                     )
 
                     # validate the result
@@ -429,6 +488,9 @@ class Module(Thread):
                     assert 'full_text' in result, "missing 'full_text' key"
                     assert 'name' in result, "missing 'name' key"
 
+                    # validate the position
+                    assert isinstance(position, int), "position is not an int"
+
                     # initialize method object
                     if my_method['name'] is None:
                         my_method['name'] = result['name']
@@ -463,11 +525,17 @@ class Module(Thread):
                         'user method {} failed ({})'.format(meth, err)
                     )
 
+            # don't be hasty mate, let's take it easy for now
+            sleep(self.config['interval'])
+
         # check and execute the 'kill' method if present
         if self.has_kill:
             try:
                 kill_method = getattr(self.module_class, 'kill')
-                kill_method(self.i3status_json, self.i3status_conf)
+                kill_method(
+                    self.i3status_thread.json_list,
+                    self.i3status_thread.config
+                )
             except Exception:
                 # this would be stupid to die on exit
                 pass
@@ -513,7 +581,7 @@ class Py3statusWrapper():
                             directories (default .i3/py3status)""")
         parser.add_argument('-n', action="store",
                             dest="interval",
-                            type=int,
+                            type=float,
                             default=config['interval'],
                             help="update interval in seconds (default 1 sec)")
         parser.add_argument('-t', action="store",
@@ -582,6 +650,10 @@ class Py3statusWrapper():
         if self.config['debug']:
             syslog(LOG_INFO, 'events thread started')
 
+        # suppress modules' ouput wrt issue #20
+        sys.stdout = open('/dev/null', 'w')
+        sys.stderr = open('/dev/null', 'w')
+
         # load and spawn modules threads
         for include_path, f_name in self.list_modules():
             try:
@@ -590,8 +662,7 @@ class Py3statusWrapper():
                     self.config,
                     include_path,
                     f_name,
-                    self.i3status_thread.config,
-                    self.i3status_thread.last_output
+                    self.i3status_thread
                 )
                 # only start and handle modules with available methods
                 if my_m.methods:
@@ -711,14 +782,14 @@ class Py3statusWrapper():
 
                 # get output from i3status
                 prefix = self.i3status_thread.last_prefix
-                json_list = self.i3status_thread.last_output
+                json_list = self.i3status_thread.json_list
 
                 # transform output from i3status
-                delta = datetime.utcnow() - self.i3status_thread.last_output_ts
+                delta = datetime.utcnow() - self.i3status_thread.json_list_ts
                 if delta.seconds > 0:
-                    json_list = self.i3status_thread.transform(delta)
+                    json_list = self.i3status_thread.adjust_time(delta)
 
-                # check and update modules threads
+                # check that every module thread is alive
                 for module in self.modules:
                     if not module.is_alive():
                         # don't spam the user with i3-nagbar warnings
@@ -728,8 +799,6 @@ class Py3statusWrapper():
                                 ','.join(module.methods.keys())
                             )
                             self.i3_nagbar(msg, level='warning')
-                        continue
-                    module.i3status_json = json_list
 
                 # construct the global output, modules first
                 if self.modules:
@@ -738,6 +807,9 @@ class Py3statusWrapper():
                 # dump the line to stdout
                 print_line('{}{}'.format(prefix, dumps(json_list)))
 
+                # update i3status output
+                self.i3status_thread.update_json_list()
+
                 # sleep a bit before doing this again
                 sleep(self.config['interval'])
             except UserWarning:
@@ -752,8 +824,10 @@ class Py3statusWrapper():
                     err = sys.exc_info()[1]
                     self.i3_nagbar('SIGUSR1 ({})'.format(err), level='warning')
 
+
 def main():
     try:
+        locale.setlocale(locale.LC_ALL, '')
         py3 = Py3statusWrapper()
         py3.setup()
     except KeyboardInterrupt:
